@@ -32,6 +32,7 @@
 #include "gst-auth.h"
 #include "gst-xml.h"
 #include "gst-marshal.h"
+#include "gst-hig-dialog.h"
 
 #ifdef GST_HAVE_ARCHIVER
 #  include <bonobo.h>
@@ -54,6 +55,7 @@
 #define BUFFER_SIZE 2048
 #define GST_DEBUG 1
 #define GST_TOOL_EOR "<!-- GST: end of request -->"
+#define REDRAW_NCHARS 50
 
 /* Define this if you want the frontend to run the backend under strace */
 /*#define GST_DEBUG_STRACE_BACKEND*/
@@ -70,6 +72,7 @@ GdkPixbuf *gentoo;
 GdkPixbuf *pld;
 GdkPixbuf *openna;
 GdkPixbuf *fedora;
+GdkPixbuf *conectiva;
 
 enum {
 	BOGUS,
@@ -170,7 +173,6 @@ report_finished_cb (GstTool *tool, GstReportLine *rline, gpointer data)
 	tool->report_finished = TRUE;
 
 	gtk_widget_hide (tool->report_window);
-	gtk_main_quit ();
 	
 	return TRUE;
 }
@@ -310,21 +312,19 @@ report_dispatch (GstTool *tool)
 }
 
 static void
-report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
+report_progress_do (GstTool *tool)
 {
-	GstTool *tool;
 	GstReportLine *rline;
 	gchar *buffer;
 	gint i = 0;
 
-	tool = GST_TOOL (data);
 	if (tool->input_block)
 		return;
 
 	if (!tool->line)
 		tool->line = g_string_new ("");
 
-	while ((buffer = gst_tool_read_from_backend (tool))) {
+	while ((buffer = gst_tool_read_line_from_backend (tool))) {
 		for (i = 0; (i < strlen (buffer)); i++) {
 			gchar c = buffer [i];
 
@@ -382,38 +382,18 @@ report_window_close_cb (GtkWidget *window, GdkEventAny *ev, gpointer data)
 }
 
 static void
-report_progress (GstTool *tool, const gchar *label)
+report_progress (GstTool *tool)
 {
-	tool->timeout_done = FALSE;
-	tool->report_list_visible = FALSE;
 	tool->report_finished = FALSE;
 	tool->report_dispatch_pending = FALSE;
 
 	gst_tool_clear_supported_platforms (tool);
 	report_clear_lines (tool);
 
-	if (label) {
-		gtk_label_set_text (GTK_LABEL (tool->report_label), label);
-		gtk_widget_show_all (tool->report_window);
-		gtk_widget_show_now (tool->report_window);
+	report_progress_do (tool);
 
-		/* This ensures the report_window will be hidden on time.
-		 * I'ts OK for long-lived directives, and short lived shouldn't
-		 * be using the report window anyways. */
-		while (gtk_events_pending ()) {
-			usleep (1);
-			gtk_main_iteration ();
-		}
-	}
-
-	tool->input_id = gtk_input_add_full (tool->backend_master_fd, GDK_INPUT_READ,
-					     report_progress_tick, NULL, tool, NULL);
-	tool->input_block = FALSE;
-
-	gtk_main ();
-
-	if (tool->input_id)
-		gtk_input_remove (tool->input_id);
+	while (gtk_events_pending ())
+		gtk_main_iteration ();
 }
 
 static GSList *
@@ -454,6 +434,7 @@ gst_tool_create_distro_images (void)
 	pld = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/pld.png", NULL);
 	openna = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/openna.png", NULL);
 	fedora = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/fedora.png", NULL);
+	conectiva = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/conectiva.png", NULL);
 }
 
 static void
@@ -523,7 +504,7 @@ gst_tool_process_startup (GstTool *tool)
 	/* let's process those startup reports. */
 	tool->report_hook_type = GST_REPORT_HOOK_LOAD;
 
-	report_progress (tool, NULL);
+	report_progress (tool);
 
 	/* stable version users should pass here only once,
 	   but if there's an inconsistency the dialog will repeat */
@@ -653,23 +634,22 @@ gst_tool_end_of_request (GString *string)
 static xmlDoc *
 gst_tool_read_xml_from_backend (GstTool *tool)
 {
-	gchar *buffer;
-	int t;
-	xmlDoc *xml;
+	xmlDoc *xml = NULL;
+	gchar *buffer = NULL;
 
 	if (!tool->xml_document)
 		return NULL;
 
-	while (! gst_tool_end_of_request (tool->xml_document)) {
-		buffer = gst_tool_read_from_backend (tool);
+	do {
+		g_free (buffer);
+		buffer = gst_tool_read_line_from_backend (tool);
 		g_string_append (tool->xml_document, buffer);
-	}
+	} while (g_strrstr (buffer, GST_TOOL_EOR "\r\n") == NULL);
 
-	if (tool->xml_document->str[0] == '<') {
+	g_free (buffer);
+
+	if (tool->xml_document->str[0] == '<')
 		xml = xmlParseDoc (tool->xml_document->str);
-	} else {
-		xml = NULL;
-	}
 
 	g_string_free (tool->xml_document, TRUE);
 	tool->xml_document = NULL;
@@ -783,7 +763,7 @@ gst_tool_run_get_directive_va (GstTool *tool, const gchar *report_sign, const gc
 		tool->xml_document = g_string_new ("");
 
 	if (location_id == NULL)
-		report_progress (tool, report_sign? _(report_sign): NULL);
+		report_progress (tool);
 
 	xml = gst_tool_read_xml_from_backend(tool);
 
@@ -826,25 +806,21 @@ gst_tool_run_set_directive_va (GstTool *tool, xmlDoc *xml,
 
 	gst_tool_send_directive (tool, directive, ap);
 
-	if (xml) {
+	if (xml)
 		gst_tool_write_xml_to_backend (tool, xml);
-		gst_tool_write_to_backend (tool, "\n");
-		gst_tool_write_to_backend (tool, GST_TOOL_EOR);
-		gst_tool_write_to_backend (tool, "\n");
-	}
 
+	/* This is tipicaly to just read the end of request string,
+	   but a set directive may return some XML too. */
+	xml_out = gst_tool_read_xml_from_backend (tool);
 
 	/* FIXME: Instead of doing the following, we should pass around a value describing
 	 * tool I/O mode (as opposed to a string to report_progress ()). */
 	tool->report_hook_type = GST_REPORT_HOOK_SAVE;
 
 	if (location_id == NULL)
-		report_progress (tool, report_sign? _(report_sign): NULL);
+		report_progress (tool);
 
-
-	/* This is tipicaly to just read the end of request string,
-	   but a set directive may return some XML too. */
-	xml_out = gst_tool_read_xml_from_backend (tool);
+	gst_tool_read_junk_from_backend (tool, GST_TOOL_EOR "\r\n");
 
 	tool->directive_running = FALSE;
 	gst_tool_idle_run_directives_add (tool);
@@ -1021,11 +997,13 @@ gst_tool_load_try (GstTool *tool)
 	if (!gst_tool_load (tool)) {
 		GtkWidget *d;
 
-		d = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
-					    GTK_BUTTONS_CLOSE,
-					    _("There was an error running the backend script,\n"
-					      "and the configuration could not be loaded."));
-
+		d = gst_hig_dialog_new (GTK_WINDOW (tool->main_dialog),
+					GTK_DIALOG_MODAL,
+					GST_HIG_MESSAGE_ERROR,
+					_("The configuration could not be loaded"),
+					_("There was an error running the backend script"),
+					GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+					NULL);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (d);
 		
@@ -1245,8 +1223,6 @@ gst_tool_type_init (GstTool *tool)
 
 	tool->glade_common_path  = g_strdup_printf ("%s/common.glade", INTERFACES_DIR);
 
-	tool->etspecs_common_path = g_strdup (ETSPECS_DIR);
-
 	tool->report_gui = xml  = gst_tool_load_glade_common (tool, "report_window");
 
 	tool->report_window     = glade_xml_get_widget (xml, "report_window");
@@ -1355,8 +1331,8 @@ gst_tool_construct (GstTool *tool, const char *name, const char *title)
 	tool->directive_queue_idle_id = 0;
 
 	/* get the remote configuration gconf keys */
-	remote_config_key = g_strjoin ("/", GST_GCONF_ROOT, "global", "remote-configuration", NULL);
-	remote_hosts_key = g_strjoin ("/", GST_GCONF_ROOT, "global", "remote-hosts-list", NULL);
+	remote_config_key = g_strjoin ("/", GST_GCONF_ROOT, "global", "remote_configuration", NULL);
+	remote_hosts_key = g_strjoin ("/", GST_GCONF_ROOT, "global", "remote_hosts_list", NULL);
 
 	do_remote_conf = gconf_client_get_bool (client, remote_config_key, &error);
 	hosts = gconf_client_get_string (client, remote_hosts_key, &error);
@@ -1364,6 +1340,9 @@ gst_tool_construct (GstTool *tool, const char *name, const char *title)
 	tool->remote_config = ((do_remote_conf) && (hosts != NULL) && (strcmp (SSH_PATH, "") != 0));
 	if (hosts != NULL) 
 		tool->remote_hosts = g_strsplit (hosts, ",", -1);
+
+	g_free (remote_config_key);
+	g_free (remote_hosts_key);
 }
 
 GstTool *
@@ -1504,18 +1483,17 @@ try_show_usage_warning (void)
 	gchar *version;
 	GConfClient *client;
 	GError *error = NULL;
-	gchar *warning_title = g_strdup_printf (_("<span weight=\"bold\" size=\"larger\">"
-						  "Welcome to the %s prerelease of the GNOME System Tools"
-						  "</span>"), VERSION);
-	gchar *warning = _("This is still a work in progress, and so it may have serious bugs. "
-			   "Due to the nature of these tools, bugs may render your computer "
-			   "<span weight=\"bold\">practically useless</span>, costing time, effort and sanity points. "
-			   "You have been warned. Thank you for trying out this prerelease of "
-			   "the GNOME System Tools!");
+	gchar *title         = g_strdup_printf (_("Welcome to the %s prerelease of the GNOME System Tools"), VERSION);
+	gchar *warning_title = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>", title);
+	gchar *warning       = _("This is still a work in progress, and so it may have serious bugs. "
+				 "Due to the nature of these tools, bugs may render your computer "
+				 "<span weight=\"bold\">practically useless</span>, costing time, effort and sanity points. "
+				 "You have been warned. Thank you for trying out this prerelease of "
+				 "the GNOME System Tools!");
 
 	client = gconf_client_get_default ();
 
-	version_key = g_strjoin ("/", GST_GCONF_ROOT, "global", "last-version", NULL);
+	version_key = g_strjoin ("/", GST_GCONF_ROOT, "global", "last_version", NULL);
 	key = g_strjoin ("/", GST_GCONF_ROOT, "global", "show_warning", NULL);
 
 
@@ -1580,6 +1558,7 @@ try_show_usage_warning (void)
 	}
 
 	g_free (warning_title);
+	g_free (title);
 	g_free (key);
 }
 
@@ -1627,34 +1606,106 @@ gst_init (const gchar *app_name, int argc, char *argv [], const poptOption optio
 	try_show_usage_warning ();
 }
 
-gchar*
-gst_tool_read_from_backend (GstTool *tool)
+static gboolean
+is_string_complete (gchar *str, GSList *list)
 {
-	gchar *buffer;
-	GString *str = g_string_new ("");
-	gchar *ptr;
-	gboolean may_exit = FALSE;
+	GSList *elem;
 	
-	buffer = (gchar*) g_malloc0 (sizeof (gchar) * BUFFER_SIZE);
+	if (strlen (str) == 0)
+		return FALSE;
 
-	do {
-		usleep (32000);
+	for (elem = list; elem; elem = g_slist_next (elem))
+		if (g_strrstr (str, elem->data) != NULL)
+			return TRUE;
+
+	return FALSE;
+}
+
+static gchar*
+gst_tool_read_from_backend_va (GstTool *tool, gchar *needle, va_list ap)
+{
+	GString *str = g_string_new ("");
+	gboolean may_exit = FALSE;
+	gint i = 0;
+	gchar c, *ptr, *arg;
+	GSList *list = NULL;
+
+	list = g_slist_prepend (list, needle);
+
+	while ((arg = va_arg (ap, char*)) != NULL)
+		list = g_slist_prepend (list, arg);
+	va_end (ap);
+
+	while (!is_string_complete (str->str, list)) {
+		c = fgetc (tool->backend_stream);
+		i++;
 		
-		while (fgets (buffer, BUFFER_SIZE, tool->backend_stream) != NULL) {
-			g_string_append (str, buffer);
-			may_exit = TRUE;
+		if (*str->str)
+			g_string_append_c (str, c);
+		else {
+			/* the string is still empty, read with O_NONBLOCK until
+			 *  it gets a char, this is done for not blocking the UI
+			 */
+			if (c != EOF) {
+				g_string_append_c (str, c);
+				fcntl (tool->backend_master_fd, F_SETFL, 0);
+			}
+			
+			usleep (500);
 		}
-		
-		while (gtk_events_pending ())
-			gtk_main_iteration ();
-	} while (!may_exit);
 
+		/* ugly hack for redrawing UI without too much overload */
+		if (i == REDRAW_NCHARS) {
+			while (gtk_events_pending ())
+				gtk_main_iteration ();
+			i = 0;
+		}
+	}
+
+	fcntl (tool->backend_master_fd, F_SETFL, O_NONBLOCK);
 	ptr = str->str;
 	g_string_free (str, FALSE);
 
 	return ptr;
 }
 
+gchar*
+gst_tool_read_from_backend (GstTool *tool, gchar *needle, ...)
+{
+	va_list ap;
+
+	va_start (ap, needle);
+	return gst_tool_read_from_backend_va (tool, needle, ap);
+}
+
+gchar*
+gst_tool_read_line_from_backend (GstTool *tool)
+{
+	gchar line[1000];
+
+	fcntl (tool->backend_master_fd, F_SETFL, 0);
+	fgets (line, 1000, tool->backend_stream);
+	fcntl (tool->backend_master_fd, F_SETFL, O_NONBLOCK);
+
+	while (gtk_events_pending ())
+		gtk_main_iteration ();
+
+	return g_strdup (line);
+}
+
+void
+gst_tool_read_junk_from_backend (GstTool *tool, gchar *needle)
+{
+	gchar *buffer = NULL;
+
+	do {
+		g_free (buffer);
+		buffer = gst_tool_read_line_from_backend (tool);
+	} while (g_strrstr (buffer, needle) == NULL);
+
+	g_free (buffer);
+}
+
 void
 gst_tool_write_xml_to_backend (GstTool *tool, xmlDoc *doc)
 {
@@ -1666,26 +1717,33 @@ gst_tool_write_xml_to_backend (GstTool *tool, xmlDoc *doc)
 	string = (gchar *) xml;
 	
 	gst_tool_write_to_backend (tool, string);
-
 	xmlFree (xml);
-
-	usleep (32000);
 }
 
 void
 gst_tool_write_to_backend (GstTool *tool, gchar *string)
 {
-	gint ntotal = 0;
 	gint nread = 0;
-	gint ret;
+	int ret;
 	gchar *p;
 
+	/* turn the descriptor blocking for writing the configuration */
+	fcntl (tool->backend_master_fd, F_SETFL, 0);
+
 	do {
 		ret = fputc (string [nread], tool->backend_stream);
 
-		if (ret >= 0)
+		if (ret != EOF)
 			nread++;
+
+		/* ugly hack for redrawing UI */
+		if (nread % REDRAW_NCHARS == 0)
+			while (gtk_events_pending ())
+				gtk_main_iteration ();
+
 	} while (nread < strlen (string));
 
 	while (fflush (tool->backend_stream) != 0);
+
+	fcntl (tool->backend_master_fd, F_SETFL, O_NONBLOCK);
 }
